/*
 ************************************************************************
void	__attribute__((interrupt,nomips16,noinline)) _Tmr2Interrupt(void);
void	init_timer1(void);
void	init_timer2(void);
void	init_vga_port();
void	init_vga(void);
void	gen_VSYNC();
void	__attribute__((interrupt,nomips16,noinline)) _Tmr1Interrupt(void);
int		get_timer1();
int		get_timer2();
void	set_timer1(int t);
void	set_timer2(int t);
 ************************************************************************
*/

/*
	PIC32MX1xx/2xx
	----------------------------------------------------------------
	Timer1:		16bit
	Timer2/3:	16bit x 2  or 32bit(pair)
	Timer4/5:	16bit x 2  or 32bit(pair)
	----------------------------------------------------------------

"VGA industry standard" 640x480 pixel mode General characteristics
	Clock frequency 25.175 MHz	
	Line  frequency 31469 Hz	(31.77uS)
	Field frequency 59.94 Hz	(16.68mS)

	Timer1 = HSync (31469 Hz)

 */
#include <p32xxxx.h>			// always in first place to avoid conflict with const.h ON
#include "utype.h"
#include "gpio.h"

#include "../spi2.h"

//#define	HSYNC_PR	(635*2+1)	//635*2	// (31.77uS * 20MHz) = 635.547
//#define	HSYNC_PR	(762*2+1)	// (31.77uS * 24MHz) = 762.5

#if	SVGA_MODE
#define	HSYNC_PR	(528*2)		// (26.4uS * 20MHz) = 528
#else
#define	HSYNC_PR	(635*2)	//635*2	// (31.77uS * 20MHz) = 635.547
#endif

volatile int g_Timer1=0;
volatile int g_Timer2=0;
volatile short g_Hcount=0;

void wait_125ns(int n);


/**********************************************************************
 *
 **********************************************************************
 */
void __attribute__((interrupt,nomips16,noinline)) _Tmr2Interrupt(void)
{
	if (IFS0bits.T2IF) {	// Timer Interrupt flag
		TMR2=0;				// reset the timer register
		IFS0CLR=(1<<9);	// IFS09: Clear the timer interrupt flag
		g_Timer2++;		// increment the counter
	}
}

/**********************************************************************
 *
 **********************************************************************
 */
// configure timer 1
void init_timer1(void)
{
//	IntConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);	// interrupt mode (interrupt.c)
//	_CP0_SET_EBASE(0xBFC00000); // Set an EBase value of 0xBFC00000	

/*
^C}16rbg񓯊JE^ [hɐݒ肷ɂ́AL̎菇sKv
܂B
1. ^C}𖳌ɂ: ONrbg(T1CON<15>) =0
2. ONbNLɂ: TCSrbg(T1CON<1>) =1
3. NbN𖳌ɂ: TSYNCrbg(T1CON<2>) =0
4. K؂ȃvXP[I
5. ^C}WX^TMR1ɒlނNA
6. vgꍇAWX^PR1ɓK؂16rbgvl
7. 荞݂gꍇ:
	a) IFSxWX^T1IF荞݃tOrbgNA
	b) IPCxWX^Ŋ荞݂̗DxƕDxݒ肷
	c) IECxWX^T1IE荞݃Cl[urbgZbg
8. ^C}Lɂ: ONrbg(T1CON<15>) =1
*/

	T1CON=0;			// reset timer 1 configuration
	TMR1=0;				// reset timer 1 counter register
#if	OVERRIDE_TIMER1_PERIOD
	PR1=50000;			// define the preload register
#else
	PR1=HSYNC_PR;		// define the preload register
#endif
	IPC1SET=7;			// select interrupt priority and sub-priority
	IFS0CLR=(1<<4);		// clear interrupt flag
	IEC0SET=(1<<4);		// enable timer 1 interrupt
	T1CON=0x8000;		// start timer 1 and set prescaler(1/1)

/*	CPU=40MHẑƂ
		TPB=50nS(20MHz) PrescalerOut=400nS(2.5MHz)
		PR1=2500Ȃ 1kHz
 		PR1=25000Ȃ 100Hz
 		PR1=50000Ȃ  50Hz
 */

/* T1CON
-----------------------------------------------------------------
bit 15 ON: Timer ONrbg(1)
		1= ^C}Lɂ
		0= ^C}𖳌ɂ
bit 14 :u0vƂēǂݏo
bit 13 SIDL: ACh[h~rbg
		1= foCXACh[hɈڍsɓ~
		0= foCXACh[hɈڍsĂp
bit 12 TWDIS: 񓯊^C}݃fBZ[u rbg
		1= ۗ̏ݓ삪܂TMR1ւ݂̏𖳎
		0= AIȏ݂\ɂ(KV[̔񓯊^C}@\)
bit 11 TWIP: 񓯊^C}ݒXe[^Xrbg
		񓯊^C}[h̏ꍇ:
		1= TMR1WX^ւ̔񓯊݂s
		0= TMR1WX^ւ̔񓯊݂͊
		^C}[h̏ꍇ:
		̃rbg͏Ɂu0vƂēǂݏo܂B
bit 10-8 :u0vƂēǂݏo
bit 7 TGATE: ^C}Q[gԐώZCl[urbg
	TCS =1̏ꍇ:
	̃rbg͖܂B
	TCS =0̏ꍇ:
	1= Q[gԐώZLɂ
	0= Q[gԐώZ𖳌ɂ
bit 6 :u0vƂēǂݏo
bit 5-4 TCKPS<1:0>: ^C}̓NbN vXP[Irbg
	11= 1:256vXP[l
	10= 1:64vXP[l
	01= 1:8vXP[l
	00= 1:1vXP[l
bit 3 :u0vƂēǂݏo
-----------------------------------------------------------------
*/
}


/**********************************************************************
 *
 **********************************************************************
 */
// configure timer 2
void init_timer2(void)
{
	T2CON=0;			// reset timer 2 configuration
	TMR2=0;				// reset timer 2 counter register
	PR2=10000;			// define the preload register

	IPC2SET=7;			// select interrupt priority and sub-priority
	IFS0CLR=(1<<9);		// clear interrupt flag
	IEC0SET=(1<<9);		// enable timer 2 interrupt
	T2CON=0x8010;		// start timer 2 and set prescaler(1/2)

/* T2CON`T5CON TCKPS<2:0> bit6-4
	111= 1:256 prescale value
	110= 1:64 prescale value
	101= 1:32 prescale value
	100= 1:16 prescale value
	011= 1:8 prescale value
	010= 1:4 prescale value
	001= 1:2 prescale value
	000= 1:1 prescale value
*/

/*	CPU=40MHẑƂ
		TPB=50nS(20MHz) PrescalerOut=400nS(2.5MHz)
 		PR2=10000Ȃ  1000Hz
 */

}

/*
VGA
  13---HSYNC--RB0
  14---VSYNC--RB1
  GREEN-------RB2
*/

#define	HSYNC	0	// PB0
#define	VSYNC	1	// PB1
#define	GREEN	2	// PB2

#define	HSYNC_pin	(PB0+HSYNC)
#define	VSYNC_pin	(PB0+VSYNC)
#define	GREEN_pin	(PB0+GREEN)

#if	1
#define	setHIGH(n)	LATBSET=(1<<(n))
#define	setLOW(n)	LATBCLR=(1<<(n))
#else
#define	setHIGH(n)	PORTBSET=(1<<(n))
#define	setLOW(n)	PORTBCLR=(1<<(n))
#endif
/**********************************************************************
 *
 **********************************************************************
 */
void init_vga_port()
{
//	ANSELBCLR=0x07;
#if	1
	TRISBCLR=0x07;
	LATBSET=0x07;
#else
	pinMode(HSYNC_pin,OUTPUT);
	pinMode(VSYNC_pin,OUTPUT);
	pinMode(GREEN_pin,OUTPUT);

	digitalWrite(HSYNC_pin,HIGH);
	digitalWrite(VSYNC_pin,HIGH);
	digitalWrite(GREEN_pin,HIGH);
#endif
}

/**********************************************************************
 *
 **********************************************************************
 */
void vsync_callback()
{
	static int i=0;
	i++;
	if(i>=30) {
		TRISBbits.TRISB15=0;
		led_toggle();
		i=0;
	}
}
/**********************************************************************
 *
 **********************************************************************
 */
void init_vga(void)
{
	init_vga_port();
#if	USE_SPI2
	SPI2_init();
	dma_init();
#endif

#if	DDS_TEST
	//
	//
#else
	init_timer1();
//	init_timer2();
#endif
}

int	DmaDoM2Spi(int cnt);
#define	dma_kick	DmaDoM2Spi
/*
One field
  2 lines front porch
  2 lines vertical sync
 25 lines back porch
  8 lines top border
480 lines video
  8 lines bottom border
---
525 lines total per field              
 */
#if	SVGA_MODE

/*
Visible area 600 15.84 
Front porch 1 0.0264 
Sync pulse 4 0.1056 
Back porch 23 0.6072 
Whole frame 628 16.5792 
*/

void gen_VSYNC()
{
	g_Hcount++;

	if(	g_Hcount < 600 ) {
#if	USE_SPI2
//		if((g_Hcount & 0x1f) < 16 ) 
		{
#if	USE_DMA
			dma_kick(g_Hcount);
#endif
		}
#else
		if((g_Hcount & 0x0f) >=8 ) {
			setHIGH(GREEN);
		}else{
			setLOW(GREEN);
		}
#endif
	}else{
#if	USE_SPI2
		setLOW(GREEN);
#endif
	}

	if(	g_Hcount==(600+1)) {
		setHIGH(VSYNC);
	}

	if(	g_Hcount==(600+1+4) ) {	// 605
		setLOW(VSYNC);
	}

	if(	g_Hcount>=(628) ){			// 666
		g_Hcount=0;
	}
}
#else
//
//	VGA @60Hz mode
//
void gen_VSYNC()
{
	g_Hcount++;

	if(	g_Hcount < 480 ) {
#if	USE_SPI2
		if((g_Hcount & 0x0f) < 8 ) {
#if	USE_DMA
			dma_kick(g_Hcount);
#endif
/*
			SPI2_write(0);
			SPI2_write(0xf0f0f0f0);
			SPI2_write(0xf0f0f0f0);
			SPI2_write(0xf0f0f0f0);
*/

		}
#else
		if((g_Hcount & 0x0f) >=8 ) {
			setHIGH(GREEN);
		}else{
			setLOW(GREEN);
		}
#endif
	}else{
#if	USE_SPI2
		setLOW(GREEN);
#endif
	}

	if(	g_Hcount==490) {
		setLOW(VSYNC);
//		vsync_callback();
		
//		led_toggle();
	}

	if(	g_Hcount==(490+2) ) {
		setHIGH(VSYNC);
	}

	if(	g_Hcount>=525) {
		g_Hcount=0;
	}
}
#endif
/**********************************************************************
 *
 **********************************************************************
 */
#if	DMA_KICK_TEST
void __attribute__((interrupt,nomips16,noinline)) _Tmr1Interrupt(void)
{
	if (IFS0bits.T1IF) {	// Timer Interrupt flag
		if((g_Timer1 & 0x3f)==0) dma_kick();
		IFS0CLR=(1<<4);	// IFS04: Clear the timer interrupt flag
		g_Timer1++;		// increment the counter
	}
}
#else
void __attribute__((interrupt,nomips16,noinline)) _Tmr1Interrupt(void)
{
	if (IFS0bits.T1IF) {	// Timer Interrupt flag
		IFS0CLR=(1<<4);	// IFS04: Clear the timer interrupt flag

#if	SVGA_MODE
		setHIGH(HSYNC);
		wait_125ns(3*8);	// 3.2uS
		setLOW(HSYNC);
#else
		setLOW(HSYNC);
		wait_125ns(3*8+7);	// 3.77uS
		setHIGH(HSYNC);
#endif

		gen_VSYNC();
//		TMR1=0;				// reset the timer register
		g_Timer1++;		// increment the counter
		
	}
}
#endif
/**********************************************************************
 *
 **********************************************************************
 */
int	 get_timer1()
{
	return g_Timer1;
}

int	 get_timer2()
{
	return g_Timer2;
}

void set_timer1(int t)
{
	g_Timer1=t;
}
void set_timer2(int t)
{
	g_Timer2=t;
}
/**********************************************************************
 *
 **********************************************************************
 */
